/*  vsa-plugin.h - header file for vsa vis and bg plugins
 *  Copyright (C) 1999  Charles <int@linuxcore.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * $Id: vsa-plugin.h,v 1.10 1999/07/31 17:09:52 int Exp $
 */

/* If you read this that probably means you're about to write a plugin.
 * remember this stuff isn't final yet and will most likely get changed
 * frequently.  suggestions appreciated.
 */

#ifndef __VSA_PLUGIN_H
#define __VSA_PLUGIN_H

#include <malloc.h>
#include <string.h>

/*
 * Plugin version
 * change the major version number when there are changes that make it
 * incompatible with older versions
 */
#define VSAP_VERSION 0x0301

#define EXPORT

#ifdef __cplusplus
extern "C" {
#endif //__cplusplus


/*
 * enumerations
 */

typedef enum {
        VSAP_EVENT_RESET,     // reset plugin to initial state
        VSAP_EVENT_PAINT,     // paint the image
	VSAP_EVENT_RESIZE,    // image size changed
	VSAP_EVENT_VARCHANGE  // dispatched immediately after a var is changed 
} VSAP_EVENT;


// the format this plugin wants the audio data in.
typedef enum {
	VSAP_FORMAT_NONE = 0, // plugin doesn't accept audio data
	VSAP_FORMAT_RAW,      // raw audio
        VSAP_FORMAT_FFT_RAW,  // raw fft output
	VSAP_FORMAT_FFT_PS,   // fft power spectrum
        VSAP_FORMAT_FFT_PCT   // scaled to width, 0-100%
} VSAP_FORMAT;



typedef enum {
	VSAP_FLAG_DUMMY = 0   // we don't have any flags yet
} VSAP_FLAGS;



// type of the plugin
typedef enum {
	VSAP_TYPE_NONE = 0,
	VSAP_TYPE_VIS,        // visualization plugin
	VSAP_TYPE_BG,         // background plugin
	VSAP_TYPE_FILTER      // filter plugin
} VSAP_TYPE;


typedef enum {
	VSAP_VTYPE_END = 0,   // special value to indicate end of list
	VSAP_VTYPE_BOOL,      // true or false (checkbox)
	VSAP_VTYPE_INT,       // integer (input field, opt. range)
	VSAP_VTYPE_FLOAT,     // float (input field, opt. range)
	VSAP_VTYPE_STRING,    // text (input field)

	VSAP_VTYPE_COLOR      // VSA_Color (color picker)
} VSAP_VTYPE;




/*
 * structures
 */


// image buffer
typedef struct {
        int width;            // image width 
        int height;           // image height
        unsigned char *data;  // image RGB data
} VSA_Image;


// generic buffer for fft/raw/etc audio data
typedef struct {
	int rate;             // sample rate
	int n_chan;           // number of channels
	int n_bits;           // number of bits / sample

	int raw_size;
	unsigned char **raw;

	int size;             // data size / channel
	void **data;          // audio data, n_chan channels
			
} VSA_Audio;

// some convenience macros
#define V_AUDIO_RAW(audio, chan)        (unsigned char *)(audio)->data[chan]
#define V_AUDIO_FFT(audio, chan)        (double *)(audio)->data[chan]
#define V_AUDIO_PCT(audio, chan)        (int *)(audio)->data[chan]



// RGB color
typedef struct {
        unsigned char r, g, b;
} VSA_Color;



/*
 * vis plugins receive this struct in paint events
 */
typedef struct {
	VSA_Image *image;
	VSA_Audio *audio;
} VSA_Data;



// vsa plugin variable
typedef union {
	int ivalue;           // integer value
	float fvalue;         // float value  
	char *svalue;         // string value
	
	VSA_Color color;      // color value
} VSA_Var;



/* plugins can define a varlist for configuration purposes.
 */
typedef struct {
	VSAP_VTYPE type;      // variable type
	char *id;             // id string
	char *parms;          // var parms, such as range
	char *desc;           // human readable description of this var
	
	void *vp;             /* optional pointer to a variable
			       * if this is NULL, get/setvar will be called
			       * use this if you don't need notification
			       * when a variable is changed
			       */
} VSA_VarDef;





// this struct should be pretty stable now...hmm or not
typedef struct {
        int vsa_version;        // set this to VSAP_VERSION

        const char *name;       // plugin name
        const char *desc;       // plugin description
        const char *author;     // plugin author
        const char *copyright;  // plugin copyright
        const char *version;    // plugin version

        VSAP_TYPE   type;
        VSAP_FORMAT format;
        VSAP_FLAGS  flags;

	// reserved
        unsigned long r1, r2, r3, r4;

} VSA_Plugin;




/*
 * some inline util functions
 * should probably move 'em to a lib if we get too many.
 */


// setup a new image buffer
static __inline__ VSA_Image *vsa_image_new(int w, int h)
{
	VSA_Image *d;

	d = (VSA_Image *)malloc(sizeof(VSA_Image));
	d->width  = w;
	d->height = h;
	d->data   = (unsigned char *)calloc(w * h,  3);

	return d;
}

static __inline__ void vsa_image_clear(VSA_Image *d)
{
	memset(d->data, '\0', d->width * d->height * 3);
}

static __inline__ void vsa_image_delete(VSA_Image *d)
{
	if(d->data)
		free(d->data);
	free(d);
}

static __inline__ int vsa_image_copy(VSA_Image *target, VSA_Image *source)
{
	if(source->width != target->width || source->height != target->height)
		return 0;
	memcpy(target->data, source->data, source->width * source->height * 3);
	return 1;
}


static __inline__ void vsa_draw_pixel_noclip(VSA_Image *d, int x, int y,
					     VSA_Color *c)
{
	register unsigned char *p;

	p = d->data + ((y * d->width) + x) * 3;
	
	*p++ = c->r;
	*p++ = c->g;
	*p   = c->b;
}

static __inline__ void vsa_draw_pixel(VSA_Image *d, int x, int y, 
				      VSA_Color *c)
{
	register unsigned char *p;

	if(x < 0 || y < 0 || x >= d->width || y >= d->height)
		return;

	p = d->data + ((y * d->width) + x) * 3;
	
	*p++ = c->r;
	*p++ = c->g;
	*p   = c->b;
}

static __inline__ int vsa_get_pixel(VSA_Image *d, int x, int y, VSA_Color *c)
{
	register unsigned char *p;

	if(x < 0 || y < 0 || x >= d->width || y >= d->height)
		return 0;

	p = d->data + ((y * d->width) + x) * 3;
	
	c->r = *p++;
	c->g = *p++;
	c->b = *p;

	return 1;
}

static __inline__ void vsa_fill_rect(VSA_Image *d, int x, int y, 
				     int w, int h, VSA_Color *c)
{
	register int a, b;

	if(w == -1)
		w = d->width;
	if(h == -1)
		h = d->height;

	for(a = 0; a < w; a++)
		for(b = 0; b < h; b++)
			vsa_draw_pixel(d, x + a, y + b, c);
}


static __inline__ void vsa_fade_colors(VSA_Color c1, VSA_Color c2,
				       VSA_Color *fade, int n_col)
{
        int n;
        float step1, step2, step3;

        step1 = (c2.r - c1.r) / (float)n_col;
        step2 = (c2.g - c1.g) / (float)n_col;
        step3 = (c2.b - c1.b) / (float)n_col;

        for(n = 0; n < n_col; n++)
        {
                fade[n].r = c1.r + (step1 * n);
                fade[n].g = c1.g + (step2 * n);
                fade[n].b = c1.b + (step3 * n);
        }
}



#ifdef __cplusplus
}
#endif //__cplusplus


#endif //__VSA_PLUGIN_H
